#include <Arduino.h>
#include <Wire.h>
#include "I2CTC31855.h"

#define  I2C_ADR                           (0x18)
#define  MAX31855_HJ_RESOLUTION            (0.25)
#define  MAX31855_CJ_RESOLUTION            (0.0625)

unsigned char i = 0;

float gethotc(unsigned char buf[]);
float getcoldc(unsigned char buf[]);

void setup() {

  // put your setup code here, to run once:
  Wire.begin();
  Wire.setClock(400000uL);
  delay(500);
  Serial.begin(9600);
  Serial.println("Start I2C-TC31855");
}

void loop() {
  // put your main code here, to run repeatedly:
  unsigned char buf[4u];
  float degc;

  if (I2CTC31855_Rd(I2C_ADR, &buf[0u], 4u)) {
    degc = gethotc(&buf[0u]);
    Serial.print("Thermocouple(deg C)=");
    Serial.print(degc, 2);
    Serial.print(" ");
    
    degc = getcoldc(&buf[2u]);
    Serial.print("Cold Junction(deg C)=");
    Serial.print(degc, 2);
    Serial.print(" ");

    if (buf[3] & 0x01) {
      Serial.print("Open Circuit ");
    }

    if (buf[3] & 0x02) {
      Serial.print("Short to GND ");
    }

    if (buf[3] & 0x04) {
      Serial.print("Short to VCC");
    }

    Serial.println(" ");

  } else {
    Serial.println("Fail reading"); 
  }
   
  delay(500);
}

float gethotc (unsigned char buf[])
{
    unsigned int hj;
    float degc;

    hj = ((unsigned int)buf[0u] << 6u) | ((unsigned int)buf[1u] >> 2u);
    
    if (hj & 0x2000u) {                                               /* test for negative                                                                    */
        hj |= 0xE000u;                                                /* mask only data                                                                       */
    }
    
    degc = (float)hj;
    degc = degc * MAX31855_HJ_RESOLUTION;                          /* degree Celsius                                                                       */

  return (degc);
}

float getcoldc (unsigned char buf[])
{
    unsigned int cj;
    float degc;

    cj = ((unsigned int)buf[0u] << 4u) | ((unsigned int)buf[1u] >> 4u); 

    if (cj & 0x0800u) {                                               /* test for negative                                                                    */
        cj |= 0xF000u;                                                /* mask only data                                                                       */
    }
    
    degc = (float)cj;
    degc = degc * MAX31855_CJ_RESOLUTION;                          /* degree Celsius                                                                       */

    return (degc);
}
